'*******************************************************************************
'  Original Program:
'  DDTSpy.bas - A simple window spy utility for investigating window and control
'               class names, handles, ID's, and parentage.
'
'               Copyright  2003 by Lance C. Edmonds.
'
'               This program code is hereby deemed Freeware.
'               No warranty is implied -- use only at your own risk!
'  Modified Program:
'  ReplaceMenu.bas - Added windows event hook to replace the default popup menu
'               on edit controls. The sample code will reverse the case on the
'               content of an edit box in another application.
'
'               March 25, 2007 by Peter Redei
'
'               Compile the program and launch it. Launch another application
'               that contains edit control. Write any text into the edit control.
'               Right click will display "Reverse" instead of the standard menu.
'               Reverse will reverse the case of the text.
'*******************************************************************************
'
#COMPILE EXE
#DIM ALL
#INCLUDE "WIN32API.INC"
'
'-------------------------------------------------------------------------------
'   ** Constants **
'-------------------------------------------------------------------------------
%IDOK           =    1
%IDCANCEL       =    2
%IDD_DIALOG1    =  101
%IDC_LABEL1     = 1000
%IDC_LABEL2     = 1001
%IDC_LABEL3     = 1002
%IDC_LABEL4     = 1003
%IDC_LABEL5     = 1004
%IDC_LABEL6     = 1005
%IDC_LABEL7     = 1006
%IDC_LABEL8     = 1007
%IDC_LABEL9     = 1008
%IDC_LABEL10    = 1009
%IDC_LABEL11    = 1010
%IDC_LABEL12    = 1011
%IDC_LABEL13    = 1012
%IDC_BUTTON1    = 1013
%IDC_BUTTON2    = 1014
%IDC_LINE1      = 1015
%IDC_LINE2      = 1016
%IDC_LINE3      = 1017
%StepTime       = 150
'-------------------------------------------------------------------------------
%IDM_RD_MENU1           = 10
%IDM_RD_MENU2           = 11
'-------------------------------------------------------------------------------
'   ** Declarations **
'-------------------------------------------------------------------------------
DECLARE CALLBACK FUNCTION ShowDIALOG1Proc()
DECLARE FUNCTION ShowDIALOG1(BYVAL hParent AS DWORD) AS LONG
'-------------------------------------------------------------------------------
GLOBAL hPopup AS LONG
GLOBAL ctrlhwnd AS DWORD
GLOBAL sContent AS STRING
GLOBAL hDlgMain AS DWORD
GLOBAL LHook AS LONG
GLOBAL xMenu AS LONG
GLOBAL yMenu AS LONG

SUB PressEscape()
    keybd_event %VK_ESCAPE, MapVirtualKey(%VK_ESCAPE, 0), 0, 0
END SUB

FUNCTION WinEventFunc(BYVAL HookHandle AS LONG, BYVAL LEvent AS LONG, BYVAL hWnd AS LONG, BYVAL idObject AS LONG, BYVAL idChild AS LONG, BYVAL idEventThread AS LONG, BYVAL dwmsEventTime AS LONG) AS LONG
    DIM className AS ASCIIZ * 64
    DIM Pt AS POINTAPI
    DIM ctrl_hwnd AS LONG

    ON ERROR RESUME NEXT
    GetClassName hWnd, className, SIZEOF(className)     'get the classname of the control that was right clicked
    Pt.x = LOINT(GetMessagePos()): xMenu = Pt.x         'get the position of the cursor
    Pt.y = HIINT(GetMessagePos()): yMenu = Pt.y
    ctrl_hwnd = WindowFromPoint(Pt.x, Pt.y)             'get the menu handle of the standard menu just about to appear


    IF GetParent(hWnd) = 0 THEN                         'act only if the parent is the desktop (as it is for standard edit menues)
        IF className = "#32768" THEN                    'act only if the classname is for standard edit menu
            PressEscape                                 'dismiss the standard menu
            SendMessage hDlgMain, %WM_CONTEXTMENU, 0, 0 'display our replacement menu instead
        END IF
    END IF

    WinEventFunc = 1
END FUNCTION

SUB HookGlobal()                                        'initiate the windows event hook
  DIM LRet AS LONG
  LHook = SetWinEventHook(%EVENT_SYSTEM_MENUPOPUPSTART, %EVENT_SYSTEM_MENUPOPUPSTART, 0&, CODEPTR(WinEventFunc), 0, 0, %WINEVENT_OUTOFCONTEXT)
END SUB

SUB UnHookGlobal()                                      'terminate the windows event hook (must be called before exit!)
    DIM LRet AS LONG
    IF LHook = 0 THEN EXIT SUB
    LRet = UnhookWinEvent(LHook)
END SUB


'-------------------------------------------------------------------------------
'   ** Main Application Entry Point **
'-------------------------------------------------------------------------------
FUNCTION PBMAIN()
    ShowDIALOG1 %HWND_DESKTOP
END FUNCTION
'-------------------------------------------------------------------------------
'
'-------------------------------------------------------------------------------
MACRO TranslateClass(szTxt)
    SELECT CASE szTxt
        CASE "#32772"
            szTxt = szTxt + " [Icon titles]"
        CASE "#32771"
            szTxt = szTxt + " [TaskSwitch]"
        CASE "#32770"
            szTxt = szTxt + " [Dialog]"
        CASE "#32769"
            szTxt = szTxt + " [Desktop]"
        CASE "#32768"
            szTxt = szTxt + " [Menu]"
        CASE ""
            szTxt = "[Desktop window]"
    END SELECT
END MACRO
'-------------------------------------------------------------------------------
FUNCTION ReverseCase(BYVAL sText AS STRING) AS STRING               'reverse the case of the text in the edit control
    DIM i AS LONG
    DIM j AS LONG
    DIM sOut AS STRING
    FOR i = 1 TO LEN(sText)
        IF MID$(sText, i, 1) >= "A" AND MID$(sText, i, 1) <= "Z" THEN
            j = ASC(MID$(sText, i, 1)) + 32
        ELSEIF MID$(sText, i, 1) >= "a" AND MID$(sText, i, 1) <= "z" THEN
            j = ASC(MID$(sText, i, 1)) - 32
        ELSE
            j = ASC(MID$(sText, i, 1))
        END IF
        sOut = sOut & CHR$(j)
    NEXT
    ReverseCase = sOut
END FUNCTION

'-------------------------------------------------------------------------------
'   ** CallBacks **
'-------------------------------------------------------------------------------
CALLBACK FUNCTION ShowDIALOG1Proc()
    STATIC lFreeze   AS LONG
    STATIC lStep     AS LONG
    STATIC sStep     AS STRING * 4
    LOCAL  Pt        AS POINTAPI
    LOCAL  hWnd      AS DWORD
    LOCAL  hWndChild AS DWORD
    LOCAL  szTxt     AS ASCIIZ * 64
    LOCAL  lID       AS WORD
    LOCAL ln         AS LONG

    SELECT CASE AS LONG CBMSG
        CASE %WM_INITDIALOG
            hDlgMain = CBHNDL                                               'store our window handle in a global variable
            MENU NEW POPUP TO hPopup                                        'create our popup menu with a single item
            MENU ADD STRING, hPopup, "&Reverse", %IDM_RD_MENU1, %MF_ENABLED '
            lStep   = 0
            sStep   = "/-\|"
            lFreeze = %FALSE
            SetTimer CBHNDL, %IDD_DIALOG1, %StepTime, 0
            HookGlobal                                                      'initiate the event hook

        CASE %WM_DESTROY
            UnhookGlobal                                                    'terminate the event hook
            IF ISFALSE lFreeze THEN
                KillTimer CBHNDL, %IDD_DIALOG1
            END IF
'
        CASE %WM_TIMER
            Pt.x = LOINT(GetMessagePos())
            Pt.y = HIINT(GetMessagePos())
            hWnd = WindowFromPoint(Pt.x, Pt.y)
            GetClassName hWnd, szTxt, SIZEOF(szTxt)
            ScreenToClient hWnd, Pt
            hWndChild = ChildWindowFromPoint(hWnd, Pt.x, Pt.y)
            IF hWndChild THEN
                hWnd = hWndChild
                szTxt = ""
                GetClassName hWnd, szTxt, SIZEOF(szTxt)
            END IF
            TranslateClass(szTxt)
            lID  = GetDlgCtrlID(hWnd)
'
            CONTROL SET TEXT CBHNDL, %IDC_LABEL3, "&&H" + HEX$(hWnd,9)
            CONTROL SET TEXT CBHNDL, %IDC_LABEL5, IIF$(lID = 65535, "-1", FORMAT$(lID))
            CONTROL SET TEXT CBHNDL, %IDC_LABEL7, szTxt

            'determine if this is an edit control
            IF INSTR(LCASE$(szTxt), "textbox") > 0 OR INSTR(LCASE$(szTxt), "edit") > 0 OR INSTR(LCASE$(szTxt), "wwg") > 0 THEN
                ln = SendMessage(hWnd, %WM_GETTEXTLENGTH, 0, 0)
                sContent = SPACE$(ln)                                       'store the content of the remote edit control in a global variable
                SendMessage hWnd, %WM_GETTEXT, 1 + ln, BYVAL STRPTR(sContent)
                ctrlhwnd = hWnd                                             'store the handle of the remote edit control in a global variable
            END IF

            hWnd = GetParent(hWnd)
            szTxt = ""
            GetClassName hWnd, szTxt, SIZEOF(szTxt)
            TranslateClass(szTxt)
            lID  = GetDlgCtrlID(hWnd)
'
            CONTROL SET TEXT CBHNDL, %IDC_LABEL9, "&&H" + HEX$(hWnd,9)
            CONTROL SET TEXT CBHNDL, %IDC_LABEL11, IIF$(lID = 65535, "-1", FORMAT$(lID))
            CONTROL SET TEXT CBHNDL, %IDC_LABEL13, szTxt
'
            lStep = (lStep + 1) MOD 4
            DIALOG SET TEXT CBHNDL, "Replace Menu " + MID$(sStep, lStep + 1, 1)
'
        CASE %WM_NCACTIVATE
            STATIC hWndSaveFocus AS DWORD
            IF ISFALSE CBWPARAM THEN
                hWndSaveFocus = GetFocus()
            ELSEIF hWndSaveFocus THEN
                SetFocus hWndSaveFocus
                hWndSaveFocus = 0
            END IF

        CASE %WM_CONTEXTMENU                                                'display our replacement popup menu
            TrackPopupMenu hPopup, %TPM_LEFTALIGN OR %TPM_RIGHTBUTTON, xMenu, yMenu, 0, CBHNDL, BYVAL 0

        CASE %WM_COMMAND
            SELECT CASE AS LONG CBCTL
                CASE %IDM_RD_MENU1                                          'upon the selection of our replacement menu reverse the text case
                    LOCAL sReversed AS STRING
                    sReversed = ReverseCase(sContent)
                    SendMessage ctrlhwnd, %WM_SETTEXT, 0, STRPTR(sReversed)

                CASE %IDOK
                    IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG = 1 THEN
                        IF ISFALSE lFreeze THEN
                            lFreeze = %TRUE
                            KillTimer CBHNDL, %IDD_DIALOG1
                            UnhookGlobal
                            CONTROL SET TEXT CBHNDL, %IDOK, "&UnFreeze"
                            DIALOG SET TEXT CBHNDL, "Replace Menu [Frozen]"
                        ELSE
                            lFreeze = %FALSE
                            HookGlobal
                            SetTimer CBHNDL, %IDD_DIALOG1, %StepTime, 0
                            CONTROL SET TEXT CBHNDL, %IDOK, "&Freeze"
                        END IF
                    END IF
'
                CASE %IDCANCEL
                    IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG = 1 THEN
                        DIALOG END CBHNDL, 0
                    END IF
'
            END SELECT
    END SELECT
END FUNCTION
'-------------------------------------------------------------------------------
'
'-------------------------------------------------------------------------------
'   ** Dialogs **
'-------------------------------------------------------------------------------
FUNCTION ShowDIALOG1(BYVAL hParent AS DWORD) AS LONG
    LOCAL hDlg AS DWORD
'
    DIALOG  NEW hParent, "Replace Menu", 150, 132, 175, 130, %WS_POPUP OR %WS_BORDER OR %WS_DLGFRAME OR %WS_CAPTION OR %WS_SYSMENU OR %WS_MINIMIZEBOX OR %WS_CLIPSIBLINGS OR %WS_VISIBLE OR %DS_MODALFRAME OR %DS_3DLOOK OR %DS_NOFAILCREATE OR %DS_SETFONT, _
        %WS_EX_WINDOWEDGE OR %WS_EX_CONTROLPARENT OR %WS_EX_TOPMOST OR %WS_EX_LEFT OR %WS_EX_LTRREADING OR %WS_EX_RIGHTSCROLLBAR, TO hDlg
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL1,  "Move mouse and check the results here:", 10, 3, 155, 10, %WS_CHILD OR %WS_VISIBLE OR %WS_BORDER OR %SS_CENTER, %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL ADD LINE,   hDlg, %IDC_LINE3,   "Line1", 0, 15, 175, 1
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL2,  "Handle", 5, 20, 60, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_RIGHT, %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL3,  "0", 70, 20, 100, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_CENTER, %WS_EX_CLIENTEDGE OR %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL SET COLOR   hDlg, %IDC_LABEL3,  %BLUE, %WHITE
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL4,  "ID", 5, 33, 60, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_RIGHT, %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL5,  "0", 70, 33, 100, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_CENTER, %WS_EX_CLIENTEDGE OR %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL SET COLOR   hDlg, %IDC_LABEL5,  %BLUE, %WHITE
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL6,  "ClassName", 5, 46, 60, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_RIGHT, %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL7,  "[none]", 70, 46, 100, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_CENTER, %WS_EX_CLIENTEDGE OR %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL SET COLOR   hDlg, %IDC_LABEL7,  %BLUE, %WHITE
    CONTROL ADD LINE,   hDlg, %IDC_LINE1,   "Line1", 0, 60, 175, 1
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL8,  "Parent's Handle", 5, 64, 60, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_RIGHT, %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL9,  "0", 70, 64, 100, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_CENTER, %WS_EX_CLIENTEDGE OR %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL SET COLOR   hDlg, %IDC_LABEL9,  %BLUE, %WHITE
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL10, "Parent's ID", 5, 77, 60, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_RIGHT, %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL11, "0", 70, 77, 100, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_CENTER, %WS_EX_CLIENTEDGE OR %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL SET COLOR   hDlg, %IDC_LABEL11, %BLUE, %WHITE
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL12, "Parent's Class", 5, 90, 60, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_RIGHT, %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL ADD LABEL,  hDlg, %IDC_LABEL13, "[none]", 70, 90, 100, 10, %WS_CHILD OR %WS_VISIBLE OR %SS_CENTER, %WS_EX_CLIENTEDGE OR %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL SET COLOR   hDlg, %IDC_LABEL13, %BLUE, %WHITE
    CONTROL ADD LINE,   hDlg, %IDC_LINE2,   "Line1", 0, 105, 175, 1
    CONTROL ADD BUTTON, hDlg, %IDOK,        "&Freeze", 30, 110, 50, 15, %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR %BS_TEXT OR %BS_DEFPUSHBUTTON OR %BS_CENTER OR %BS_VCENTER, %WS_EX_LEFT OR %WS_EX_LTRREADING
    CONTROL ADD BUTTON, hDlg, %IDCANCEL,    "E&xit", 95, 110, 50, 15
'
    DIALOG POST hDlg,   %DM_SETDEFID,       %IDOK, 0
    DIALOG SHOW MODAL hDlg, CALL ShowDIALOG1Proc
END FUNCTION
